using Overdash.Player;
using UnityEngine;

namespace Overdash.Combat.Projectiles
{
    public class ExplosiveProjectile : ProjectileBase
    {
        [SerializeField] private float explosionRadius;

        public AudioClip explosionSound;
        protected override void PlayerHit(PlayerCharacter player, Collision2D collision)
        {
            if (player.GetComponent<PlayerMovement>().dashing)
            {
                player.GetComponent<PlayerCombat>().AddAmmunition(ammoType);
                Destroy(gameObject);
            }
            else
            {
                CreateExplosion();
            }
        }

        protected override void WallHit(Collision2D _)
        {
            CreateExplosion();
            GetComponent<Collider2D>().enabled = false;
            GetComponent<Rigidbody2D>().bodyType = RigidbodyType2D.Static;
        }

        protected override void Update()
        {
            if (isActive && GetComponent<Rigidbody2D>().bodyType != RigidbodyType2D.Static)
                rigidbody.velocity = transform.up * ammoType.projectileSpeed;
        }

        protected void CreateExplosion()
        {

            GetComponent<AudioSource>().PlayOneShot(explosionSound);

           SpriteRenderer[] children = GetComponentsInChildren<SpriteRenderer>();
            foreach (SpriteRenderer child in children)
            {
                if (child.gameObject.name == "Explosion")
                {
                    child.enabled = true;
                    child.GetComponent<Animator>().SetTrigger("Explode");
                    Debug.Log("Explosion animation started");
                    float animationTime = child.GetComponent<Animator>().GetCurrentAnimatorStateInfo(0).length;
                    Destroy(gameObject, animationTime);
                }
                else
                    child.enabled = false;
            }
            Collider2D[] colliders = Physics2D.OverlapCircleAll(transform.position, explosionRadius);
            foreach (Collider2D nearbyObject in colliders)
            {
                if (nearbyObject.gameObject.TryGetComponent(out PlayerCharacter player))
                {
                    player.Kill(shooter);
                    InvokeOnHitPlayer(new ProjectilePlayerHit
                    { 
                        Shooter = shooter,
                        ProjectileBase = this
                        //Collision = collision  // Collision by sa dalo vyrobit, ale naco?
                    });
                }
            }
        }
        private void OnDrawGizmos()
        {
            Gizmos.DrawWireSphere(transform.position, explosionRadius);
        }
    }
}