using System.Collections;
using Overdash.Player;
using UnityEngine;

namespace Overdash.Combat.Projectiles
{
    public class SmokeProjectile : ProjectileBase
    {
        [SerializeField] private float smokeRadius;
        [SerializeField] private float smokeDuration;
        [SerializeField] private float spreadTime;
        
        private bool exploded = false;
        private bool disappearing = false;
        private Vector3 originalScale;
        private float spreadTimer = 0f;
        private GameObject smoke;
        public AudioClip smokeSound;
        protected override void PlayerHit(PlayerCharacter player, Collision2D collision)
        {
            if (player.GetComponent<PlayerMovement>().dashing)
            {
                player.GetComponent<PlayerCombat>().AddAmmunition(ammoType);
                Destroy(gameObject);
            }
            else
            {
                CreateExplosion();
            }
        }

        protected override void WallHit(Collision2D _)
        {
            CreateExplosion();
            GetComponent<Collider2D>().enabled = false;
            GetComponent<Rigidbody2D>().bodyType = RigidbodyType2D.Static;
        }

        protected override void Update()
        {
            if (isActive && GetComponent<Rigidbody2D>().bodyType != RigidbodyType2D.Static)
                rigidbody.velocity = transform.up * ammoType.projectileSpeed;
            //move aplha to 0 over time
            if (disappearing)
            {
                exploded = false;
                smoke.gameObject.GetComponent<SpriteRenderer>().color = new Color(1, 1, 1, Mathf.Lerp(1, 0, spreadTimer / spreadTime));
                spreadTimer += Time.deltaTime;
            }
            //scale up the smoke to desired radius over time
            if (exploded)
            {
                transform.localScale = Vector3.Lerp(originalScale * 0.1f, originalScale * smokeRadius, spreadTimer / spreadTime);
                spreadTimer += Time.deltaTime;
            }
        }

        protected void CreateExplosion()
        {
            GetComponent<AudioSource>().PlayOneShot(smokeSound);
            SpriteRenderer[] children = GetComponentsInChildren<SpriteRenderer>();
            foreach (SpriteRenderer child in children)
            {
                if (child.gameObject.name == "Explosion")
                {
                    child.enabled = true;
                    child.GetComponent<Animator>().SetTrigger("Smoke");
                    smoke = child.gameObject;
                    StartCoroutine(Disappear());
                }
                else
                    child.enabled = false;
            }
            exploded = true;
            originalScale = transform.localScale;
        }
        private void OnDrawGizmos()
        {
            Gizmos.DrawWireSphere(transform.position, smokeRadius);
        }
        private IEnumerator Disappear()
        {
            yield return new WaitForSeconds(smokeDuration);
            spreadTimer = 0f;
            disappearing = true;
            Destroy(gameObject, spreadTime);
        }
    }
}