using UnityEngine;

namespace Overdash.General
{
    [RequireComponent(typeof(Camera))]
    public class FixedAspectRatio : MonoBehaviour
    {
        [SerializeField] private float targetAspectRatio = 16f / 9f;

        private void Awake() => UpdateAspectRatio();
        private void Update() => UpdateAspectRatio();

        private void UpdateAspectRatio()
        {
            var camera = GetComponent<Camera>();
            var currentAspectRatio = (float)Screen.width / Screen.height;
            var scaleRatio = currentAspectRatio / targetAspectRatio;
        
            if (scaleRatio < 1f)
                camera.rect = new Rect(0, (1f - scaleRatio) / 2f, 1f, scaleRatio);
            else
                camera.rect = new Rect((1f - 1f / scaleRatio) / 2f, 0, 1f / scaleRatio, 1f);
        }
    }
}