using System;
using System.Collections;
using System.Collections.Generic;
using Overdash.Common;
using Overdash.Levels.Modifiers;
using Overdash.Player;
using Overdash.UI;
using UnityEngine;
using Random = UnityEngine.Random;

namespace Overdash.Global
{
    public class RoundLoop : MonoBehaviourSingleton<RoundLoop>
    {
        public enum State { StartCountdown, InProgress, EndScreen, }
        public static State state { get; private set; }
        
        public static event Action OnNewRound;
        public static event Action OnRoundStart;
        public static event Action OnRoundEnd;
        
        private static readonly List<PlayerDeathInfo> RoundHistory = new();

        public static IRoundModifier currentRoundModifier { get; private set; }
        private static readonly List<IRoundModifier> RoundModifiers = new() { new LowGravity(),    new SlowMo(),    new SpeedUp(),     new InfiniteAmmo(), 
                                                                              new InfiniteJumps(), new NoDashing(), new OneBullet()   };
        
        // UI Elements
        private static RoundStartCountdown _roundStartCountdown;
        private static EndOfRoundScreen _endOfRoundScreen;

        
        protected override void Awake()
        {
            base.Awake();
            _endOfRoundScreen = FindObjectOfType<EndOfRoundScreen>();
            _roundStartCountdown = FindObjectOfType<RoundStartCountdown>();
            PlayerManager.OnPlayerKilled += OnPlayerKilled;
        }

        public static void StartRound()
        {   
            // Pick random modifier
            currentRoundModifier = Random.value < 0.5f 
                                      ? RoundModifiers[Random.Range(0, RoundModifiers.Count)] 
                                      : null;
            currentRoundModifier?.ApplyModifier();
            
            state = State.StartCountdown;
            OnNewRound?.Invoke();
            instance.StartCoroutine(StartRoundRoutine());
        }
        
        private static IEnumerator StartRoundRoutine()
        {
            yield return _roundStartCountdown.Countdown();
            
            state = State.InProgress;
            OnRoundStart?.Invoke();
        }
        
        private static void OnPlayerKilled(PlayerDeathInfo deathInfo)
        {
            RoundHistory.Add(deathInfo);
            var alivePlayers = PlayerManager.alivePlayers.Count;
            if (alivePlayers == 1 || alivePlayers == 0 && PlayerManager.players.Count == 1)
                instance.StartCoroutine(EndRoundRoutine());
        }
        
        private static IEnumerator EndRoundRoutine()
        {
            state = State.EndScreen;
            yield return new WaitForSeconds(2f);
            Time.timeScale = 0f;
            yield return _endOfRoundScreen.Run(RoundHistory);
            RoundHistory.Clear();
            currentRoundModifier?.RemoveModifier();
            OnRoundEnd?.Invoke();
            Time.timeScale = 1f;
            StartRound();
        }
    }
}