using System.Collections.Generic;
using Overdash.Combat;
using UnityEngine;

namespace Overdash.Player
{
    [RequireComponent(typeof(AudioSource))]
    public class PlayerSound : MonoBehaviour
    {
        [SerializeField] private GameObject PickupField;
        [SerializeField] private AudioSource SlidePlayer;
        [SerializeField] private AudioSource audioSource;

        public AudioClip pickupSound;
        public AudioClip dashSound;
        public AudioClip slideSound;
        public AudioClip noAmmoSound;
        public List<AudioClip> jumpSounds_ground = new();
        public List<AudioClip> jumpSounds_air= new();
        public List<AudioClip> landSounds = new();
        public List<AudioClip> shotSounds = new();

        

        void Start()
        {
            audioSource = GetComponent<AudioSource>();
            PlayerMovement playerMovement = GetComponent<PlayerMovement>();
            playerMovement.OnJumpGround += OnJump_ground;
            playerMovement.OnJumpAir += OnJump_air;
            playerMovement.OnHitGround += OnLand;
            playerMovement.OnDash += OnDash;
            playerMovement.OnSlide += OnSlide;
            playerMovement.OnEndSlide += OnEndSlide;
            GetComponent<PlayerCombat>().OnShootEmptyClip += () => audioSource.PlayOneShot(noAmmoSound);

            SlidePlayer.clip = slideSound;

            GetComponent<PlayerCombat>().OnShoot += OnShoot;
            //PickupField.GetComponent<PlayerPickupField>().OnPickup += OnPickup; doesnt play sound when dash pickup
            GetComponent<PlayerCombat>().OnAmmunitionAdded += OnPickup;
        }

        private void OnJump_ground()
        {
            if (jumpSounds_ground.Count == 0) return;
            audioSource.PlayOneShot(jumpSounds_ground[Random.Range(0, jumpSounds_ground.Count)]);
        }
        private void OnJump_air()
        {
            if (jumpSounds_air.Count == 0) return;
            audioSource.PlayOneShot(jumpSounds_air[Random.Range(0, jumpSounds_air.Count)]);
        }
        private void OnLand()
        {
            if (landSounds.Count == 0) return;
            audioSource.PlayOneShot(landSounds[Random.Range(0, landSounds.Count)]);
        }
        private void OnDash()
        {
            if (dashSound == null) return;
            audioSource.PlayOneShot(dashSound);

        }
        private void OnPickup(AmmoType _)
        {
            if (pickupSound == null) return;
            audioSource.PlayOneShot(pickupSound);

        }
        private void OnShoot(AmmoType _)
        {
            if (shotSounds.Count == 0) return;
                audioSource.PlayOneShot(shotSounds[Random.Range(0, shotSounds.Count)]);
        }
        private void OnSlide()
        {
            if (slideSound == null) return;
            SlidePlayer.Play(0);
        }
        private void OnEndSlide()
        {
            if (slideSound == null) return;
            SlidePlayer.Stop();
        }
    }
}