﻿using System.Collections.Generic;
using Overdash.Player;
using UnityEngine;
using UnityEngine.UI;

namespace Overdash.UI
{
    public class PlayerAmmoCounter : MonoBehaviour
    {
        // Settings
        [SerializeField] private float nextAmmoSize = 10f;
        [SerializeField] private float otherAmmoSize = 5f;
        
        private readonly List<Image> ammoIcons = new ();
        private PlayerCombat playerCombat;
        
        private void Awake()
        {
            playerCombat = GetComponentInParent<PlayerCombat>();
            
            playerCombat.OnShoot += _ => UpdateCount();
            playerCombat.OnAmmunitionAdded += _ => UpdateCount();
        }

        private void Start() => UpdateCount();

        private void UpdateCount()
        { 
            // Clear existing icons
            ammoIcons.ForEach(x => Destroy(x.gameObject));
            ammoIcons.Clear();
            
            foreach (var ammo in playerCombat.Ammunition)
            {
                // Create icon
                var icon = new GameObject("AmmoIcon", typeof(Image)).GetComponent<Image>();
                icon.transform.SetParent(transform);
                icon.transform.localScale = Vector3.one;
                icon.color = ammo.color;
                icon.sprite = ammo.pickupPrefab.GetComponentInChildren<SpriteRenderer>().sprite;
                icon.rectTransform.sizeDelta = ammoIcons.Count == 0 ? new Vector2(nextAmmoSize, nextAmmoSize) : 
                                                                      new Vector2(otherAmmoSize, otherAmmoSize);
                ammoIcons.Add(icon);
            }
        }
    }
}