using System;
using System.Collections;
using System.Collections.Generic;
using TMPro;
using UnityEngine;
using Random = UnityEngine.Random;

namespace Overdash.UI
{
    [RequireComponent(typeof(TextMeshProUGUI))]
    public class RoundStartCountdown : MonoBehaviour
    {
        [field: SerializeField] public int countdownSeconds { get; private set; } = 3;
        [SerializeField] public float countdownFontSize = 1.25f;
        [SerializeField] public float countdownFinishedFontSize = 3f;
        
        public event Action OnCountdownStarted;
        public event Action OnCountdownFinished;
        
        private readonly List<string> countdownFinishedTexts = new()
        {
            "fight", "battle", "commence", "go", "start", 
            "begin", "mayhem", "rumble", "brawl", "clash", 
            "action", "engage", "havoc", "chaos", "carnage" 
        };
        
        private TextMeshProUGUI textComponent;
        
        private void Awake()
        {
            textComponent = GetComponent<TextMeshProUGUI>();
        }
        
        public IEnumerator Countdown()
        {
            OnCountdownStarted?.Invoke();
            
            textComponent.color = Color.yellow;
            textComponent.fontSize = countdownFontSize;
            for (var i = countdownSeconds; i > 0; i--)
            {
                textComponent.text = i.ToString();
                yield return new WaitForSecondsRealtime(0.75f);
            }
            
            textComponent.fontSize = countdownFinishedFontSize;
            textComponent.text = countdownFinishedTexts[Random.Range(0, countdownFinishedTexts.Count)].ToUpper() + "!";
            OnCountdownFinished?.Invoke();
            StartCoroutine(FadeOut());
        }
        
        private IEnumerator FadeOut()
        {
            yield return new WaitForSecondsRealtime(0.75f);
            const float fadeTime = 0.75f;
            for (var t = 0f; t < fadeTime; t += Time.unscaledDeltaTime)
            {
                textComponent.color = Color.Lerp(Color.yellow, Color.clear, t / fadeTime);
                yield return null;
            }
            textComponent.color = Color.clear;
        }
    }
}
